from typing import Dict, List
import click
from docker import APIClient, errors
import dockerpty
from freshenv.view import count_environents
from rich import print
from requests import exceptions
from freshenv.console import console
from os import getcwd, path

client = APIClient(base_url="unix://var/run/docker.sock")
dir = getcwd()
folder = path.basename(dir)

def create_environment(flavour: str, command: str, ports: List[int], name: str) -> Dict:
    if name == "index":
        name = str(count_environents() + 1)
    
    container = client.create_container(
        name=f"freshenv_{name}",
        image=f"ghcr.io/raiyanyahya/{flavour}/{flavour}",
        stdin_open=True,
        tty=True,
        command=command,
        ports=ports,
        volumes=["/home/devuser"],
        host_config=client.create_host_config(binds=[
        f"{dir}:/home/devuser/{folder}:delegated",
        "/home/$USER/.gitconfig:/root/.gitconfig:ro",
        "/home/$USER/.ssh:/root/.ssh:ro",
        "/var/run/docker.sock:/var/run/docker.sock"
    ])
    )
    return container


def pull_and_try_again(flavour, command, ports, name):
    try:
        with console.status("Flavour doesnt exist locally. Fetching flavour...", spinner="arrow2"):
            client.pull(f"ghcr.io/raiyanyahya/{flavour}/{flavour}")
        container = create_environment(flavour, command, ports, name)
        dockerpty.start(client, container)
    except (errors.ImageNotFound, exceptions.HTTPError):
        print(":x: flavour doesnt exist")


@click.command("provision")
@click.option("--flavour","-f",default="devenv",help="The flavour of the environment.",show_default=True)
@click.option("--command","-c",default="zsh",help="The command to execute at startup of environment.",show_default=True)
@click.option("--ports","-p", default=[3000], help="List of ports to forward.", show_default=True)
@click.option("--name","-n", default="index", help="Name of your environment.", show_default=False)
def provision(flavour: str, command: str, ports: List[int], name: str) -> None:
    """Provision a developer environment."""
    try:
        container = create_environment(flavour, command, ports, name)
        dockerpty.start(client, container)
    except (exceptions.HTTPError, errors.NotFound):
        pull_and_try_again(flavour, command, ports, name)
    except Exception as e:
        print("Unknown exception: {}".format(e))



import click
from docker import APIClient
import dockerpty
from rich import print

client = APIClient(base_url="unix://var/run/docker.sock")


@click.command("start")
@click.option("--name", "-n",required=True,help="Name of your environment to resume.")
def start(name: str) -> None:
    """Resume working in an environment."""
    containers = client.containers(all=True, filters={"name": name})
    if containers:
        dockerpty.start(client, containers[0])
    else:
        print(f":ghost: No freshenv environment called [bold]{name}[/bold] found.")



from typing import Dict, List
import click
from docker import APIClient
from rich import print

client = APIClient(base_url="unix://var/run/docker.sock")


def count_environents() -> int:
    return len(get_list_environments())


def get_list_environments() -> List[Dict]:
    environment_list = []
    try:
        environment_list = client.containers(all=True,filters={"label": "maintainer=Raiyan Yahya <raiyanyahyadeveloper@gmail.com>"})
    except Exception as error:
        print("Unknown exception: {}".format(error))
    return environment_list


@click.command("view")
def view() -> None:
    """View local freshenv managed environments."""
    container_list = get_list_environments()
    if not container_list:
        print(":computer: No freshenv environments found.")
    for container in container_list:
        if "Exited" in container.get("Status"):  # type: ignore
            img = ":arrow_down_small: "
        else:
            img = ":arrow_up_small: "
        print(
            img,
            "Name: [bold blue]" + container.get("Names")[0] + "[/bold blue]",  # type: ignore
            "| Flavour: [bold blue]" + container.get("Image").split("/")[-1] + "[/bold blue]",  # type: ignore
            "| State: [bold blue]" + container.get("Status") + "[/bold blue]",  # type: ignore
        )



from os.path import abspath, dirname, join, isfile
from os import environ
from setuptools import find_packages, setup
import sys
version_f = ".version"
this_dir = abspath(dirname(__file__))
with open(join(this_dir, "README.md"), encoding="utf-8") as file:
    long_description = file.read()


def get_version():
    if isfile(version_f):
        with open(version_f) as version_file:
            version = version_file.read().strip()
            return version
    elif (
        "build" in sys.argv
        or "egg_info" in sys.argv
        or "sdist" in sys.argv
        or "bdist_wheel" in sys.argv
    ):
        version = environ.get("VERSION", "0.0")  # Avoid PEP 440 warning
        if "-SNAPSHOT" in version:
            version = version.replace("-SNAPSHOT", ".0")
        with open(version_f, "w+") as version_file:
            version_file.write(version)
        return version


setup(
    name="freshenv",
    python_requires=">3.6",
    options={"bdist_wheel": {"universal": "1"}},
    version=get_version(),
    description="A cli to provision and manage local developer environments.",
    long_description="A cli to provision and manage local developer environments.",
    url="https://github.com/raiyanyahya/freshenv",
    author="Raiyan Yahya",
    author_email="raiyanyahyadeveloper@gmail.com",
    keywords=["cli","developer-tools","productivity"],
    packages=find_packages(),
    install_requires=["click==7.1.2", "docker==4.3.1", "rich==10.16.1", "dockerpty==0.4.1"],
    entry_points={"console_scripts": ["freshenv=freshenv.cli:cli"]},
    tests_require=["mock >= 2.0.0"],
)



import click
from docker import APIClient, errors
from rich import print

client = APIClient(base_url="unix://var/run/docker.sock")


@click.command("remove")
@click.option("--name","-n",required=True,help="Name of your environment to remove.")
@click.option('--force', '-f', is_flag=True, help="Force remove an environment.")
def remove(name: str, force: bool) -> None:
    """Remove a freshenv environment."""
    try:
        client.remove_container(container=name, force=force)
        print(f":boom: {name} environment removed.")
    except errors.NotFound:
        print(f":ghost: No freshenv environment called [bold]{name}[/bold] found.")
    except errors.APIError as e:
        if e.status_code == 409:
            print(f":runner: {name} is a [bold green]running[/bold green] environment. Close the session first or use the [bold blue]--force[/bold blue] flag.")
        else:
            raise Exception(e)
    except Exception as e:
        print("Unknown exception: {}".format(e))
